@@ -9,6 +9,7 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <cfenv>
 
 
 namespace capitest {
@@ -19,6 +20,7 @@ namespace capitest {
         GEOSGeometry* geom2_ = nullptr;
         GEOSGeometry* geom3_ = nullptr;
         GEOSGeometry* input_ = nullptr;
+        GEOSGeometry* result_ = nullptr;
         GEOSGeometry* expected_ = nullptr;
         char* wkt_ = nullptr;
 
@@ -28,6 +30,8 @@ namespace capitest {
             wktw_ = GEOSWKTWriter_create();
             GEOSWKTWriter_setTrim(wktw_, 1);
             GEOSWKTWriter_setRoundingPrecision(wktw_, 10);
+
+            std::feclearexcept(FE_ALL_EXCEPT);
         }
 
         ~utility()
@@ -46,6 +50,9 @@ namespace capitest {
             if (input_) {
                 GEOSGeom_destroy(input_);
             }
+            if (result_) {
+                GEOSGeom_destroy(result_);
+            }
             if (expected_) {
                 GEOSGeom_destroy(expected_);
             }
