@@ -7,13 +7,96 @@
 #include <geos/geom/Envelope.h>
 #include <geos/geom/Coordinate.h>
 
+#include <array>
+#include <cfenv>
+
 namespace tut {
 //
 // Test Group
 //
 
+using geos::geom::Coordinate;
+using geos::geom::Envelope;
+
 // dummy data, not used
-struct test_envelope_data {};
+struct test_envelope_data {
+    test_envelope_data()
+    {
+        std::feclearexcept(FE_ALL_EXCEPT);
+    }
+
+    static std::size_t
+    absdiff(std::size_t a, std::size_t b)
+    {
+        auto x = std::minmax(a, b);
+        return x.second - x.first;
+    }
+
+    static void
+    check_intersects(const Envelope& e1, const Envelope& e2, bool expected)
+    {
+        ensure_equals(e1.intersects(e2), expected);
+        ensure_equals(e1.intersects(&e2), expected);
+        ensure_equals(e1.disjoint(e2), !expected);
+        ensure_equals(e1.disjoint(&e2), !expected);
+
+        ensure_equals(e2.intersects(e1), expected);
+        ensure_equals(e2.intersects(&e1), expected);
+        ensure_equals(e2.disjoint(e1), !expected);
+        ensure_equals(e2.disjoint(&e1), !expected);
+
+        Coordinate p0, p1, q0, q1;
+
+        if (!e2.isNull()) {
+            q0 = {e2.getMinX(), e2.getMaxY()};
+            q1 = {e2.getMaxX(), e2.getMinY()};
+            ensure_equals(e1.intersects(q0, q1), expected);
+            ensure_equals(e1.intersects(q1, q0), expected);
+        }
+
+        if (!e1.isNull()) {
+            p0 = {e1.getMinX(), e1.getMinY()};
+            p1 = {e1.getMaxX(), e1.getMaxY()};
+
+            ensure_equals(e2.intersects(p0, p1), expected);
+            ensure_equals(e2.intersects(p1, p0), expected);
+        }
+
+        if (!e1.isNull() && !e2.isNull()) {
+            ensure_equals(Envelope::intersects(p0, p1, q0, q1), expected);
+            ensure_equals(Envelope::intersects(p1, p0, q0, q1), expected);
+            ensure_equals(Envelope::intersects(p1, p0, q1, q0), expected);
+            ensure_equals(Envelope::intersects(p1, p0, q0, q1), expected);
+        }
+    }
+
+    static void
+    check_intersects(const Envelope& e1, const Coordinate& q, bool expected)
+    {
+        ensure_equals(e1.intersects(q), expected);
+        ensure_equals(e1.intersects(q.x, q.y), expected);
+        ensure_equals(e1.contains(q), expected);
+        ensure_equals(e1.contains(q.x, q.y), expected);
+        ensure_equals(e1.covers(&q), expected);
+        ensure_equals(e1.covers(q.x, q.y), expected);
+
+        if (!e1.isNull()) {
+            Coordinate p0{e1.getMinX(), e1.getMinY()};
+            Coordinate p1{e1.getMaxX(), e1.getMaxY()};
+            ensure_equals(Envelope::intersects(p0, p1, q), expected);
+        }
+    }
+
+    static void
+    ensure_no_fp_except()
+    {
+        ensure("FE_DIVBYZERO raised", !std::fetestexcept(FE_DIVBYZERO));
+        //ensure("FE_INEXACT raised", !std::fetestexcept(FE_INEXACT));
+        ensure("FE_INVALID raised", !std::fetestexcept(FE_INVALID));
+        ensure("FE_OVERFLOW raised", !std::fetestexcept(FE_OVERFLOW));
+        ensure("FE_UNDERFLOW raised", !std::fetestexcept(FE_UNDERFLOW));
+    }
+};
 
 typedef test_group<test_envelope_data> group;
 typedef group::object object;
@@ -36,6 +119,8 @@ void object::test<1>
 
     ensure_equals(empty.getWidth(), 0);
     ensure_equals(empty.getHeight(), 0);
+
+    ensure_no_fp_except();
 }
 
 // 2 - Test of overridden constructor
@@ -57,6 +142,8 @@ void object::test<2>
     ensure_equals(box.getMaxX(), box.getMaxY());
 
     ensure_equals(box.getWidth(), box.getHeight());
+
+    ensure_no_fp_except();
 }
 
 // 3 - Test of copy constructor
@@ -75,6 +162,8 @@ void object::test<3>
     ensure(!copied.isNull());
     ensure(copied == box);
     ensure_equals(copied.getWidth(), copied.getHeight());
+
+    ensure_no_fp_except();
 }
 
 // 4 - Test of setToNull()
@@ -88,6 +177,8 @@ void object::test<4>
     ensure(!e.isNull());
     e.setToNull();
     ensure(e.isNull());
+
+    ensure_no_fp_except();
 }
 
 // 5 - Test of equals()
@@ -100,6 +191,7 @@ void object::test<5>
     geos::geom::Envelope zero(0, 0, 0, 0);
     geos::geom::Envelope zero2(0, 0, 0, 0);
     geos::geom::Envelope box(0, 100, 0, 100);
+    geos::geom::Envelope inf(0, 100, 0, std::numeric_limits<double>::infinity());
 
     ensure(empty.isNull());
     ensure(!zero.isNull());
@@ -117,6 +209,10 @@ void object::test<5>
 
     ensure(!box.equals(&empty));
     ensure(!box.equals(&zero));
+
+    ensure(!inf.isfinite());
+
+    ensure_no_fp_except();
 }
 
 // 6 - Test of contains()
@@ -148,18 +244,14 @@ void object::test<6>
     ensure("big envelope contains small envelope", big.contains(small));
     ensure("non-empty envelope contains itself", big.contains(big));
 
-    // Test raw point
-    ensure(small.contains(0, 0));
-    ensure(small.contains(-1, -1));
-    ensure(!small.contains(5, 5));
+    // Test points
+    check_intersects(small, {0, 0}, true);
+    check_intersects(small, {-1, -1}, true);
+    check_intersects(small, {5, 5}, false);
 
-    // Test coordinate
-    geos::geom::Coordinate origin(0, 0, 0);
+    check_intersects(empty, {0, 0}, false);
 
-    ensure_equals(origin.x, 0);
-    ensure_equals(origin.y, 0);
-    ensure_equals(origin.z, 0);
-    ensure(small.contains(origin));
+    ensure_no_fp_except();
 }
 
 // Test of intersects() and disjoint()
@@ -168,54 +260,55 @@ template<>
 void object::test<7>
 ()
 {
+    constexpr std::size_t nrow = 3;
+    constexpr std::size_t ncol = 3;
+
+    std::array<std::array<Envelope, ncol>, nrow> envelopes;
+
+    double xmin = 0.0;
+    double xmax = 1.0;
+    double ymin = 0.0;
+    double ymax = 2.0;
+    double dx = (xmax - xmin) / static_cast<double>(ncol);
+    double dy = (ymax - ymin) / static_cast<double>(nrow);
+    for (std::size_t i = 0; i < ncol; i++) {
+        for (std::size_t j = 0; j < nrow; j++) {
+            double x0 = xmin + static_cast<double>(i)*dx;
+            double x1 = xmin + static_cast<double>(i + 1)*dx;
+            double y0 = ymax - static_cast<double>(j + 1)*dy;
+            double y1 =	ymax - static_cast<double>(j)*dy;
+
+            envelopes[i][j] = Envelope(x0, x1, y0, y1);
+        }
+    }
+
     geos::geom::Envelope empty;
-    geos::geom::Envelope with_origin(-100, 100, -100, 100);
-    geos::geom::Envelope moved(50, 150, 50, 150);
 
-    ensure(empty.isNull());
-    ensure(!with_origin.isNull());
-    ensure(!moved.isNull());
+    // check intersection against empty
+    for (std::size_t ia = 0; ia < ncol; ia++) {
+        for (std::size_t ja = 0; ja < nrow; ja++) {
+            check_intersects(envelopes[ia][ja], empty, false);
+        }
+    }
 
-    // Test empty envelope by reference
-    ensure("empty envelope does not intersect non-empty envelope", !empty.intersects(with_origin));
-    ensure("non-empty envelope does not intersect empty envelope", !with_origin.intersects(empty));
-    ensure("empty envelope is disjoint with non-empty envelope", empty.disjoint(with_origin));
-    ensure("non-empty envelope is disjoint with empty envelope", with_origin.disjoint(empty));
+    // check intersection against adjacent
+    for (std::size_t ia = 0; ia < ncol; ia++) {
+        for (std::size_t ja = 0; ja < nrow; ja++) {
+            for (std::size_t ib = 0; ib < ncol; ib++) {
+                for (std::size_t jb = 0; jb < nrow; jb++) {
+                    bool should_intersect = absdiff(ia, ib) <= 1 && absdiff(ja, jb) <= 1;
 
-    // Test empty envelope by pointer
-    ensure("empty envelope does not intersect non-empty envelope", !empty.intersects(&with_origin));
-    ensure("non-empty envelope does not intersect empty envelope", !with_origin.intersects(&empty));
-    ensure("empty envelope is disjoint with non-empty envelope", empty.disjoint(&with_origin));
-    ensure("non-empty envelope is disjoint with empty envelope", with_origin.disjoint(&empty));
+                    check_intersects(envelopes[ia][ja], envelopes[ib][jb], should_intersect);
+                }
+            }
+        }
+    }
 
-    // Test empty envelope does not intersect self
-    ensure("empty envelope does not intersect self", !empty.intersects(empty));
-    ensure("empty envelope is disjoint with self", empty.disjoint(empty));
 
-    // Test non-empty envelope by reference
-    ensure(with_origin.intersects(moved));
-    ensure(!with_origin.disjoint(moved));
-
-    ensure(moved.intersects(with_origin));
-    ensure(!moved.disjoint(with_origin));
-
-    // Test intersection with raw point
-    ensure(with_origin.intersects(0, 0));
-    ensure(with_origin.intersects(-100, 100));
-    ensure(!with_origin.intersects(-200, 200));
-
-    // Test intersection with coordinate
-    geos::geom::Coordinate origin(0, 0, 0);
-
-    ensure_equals(origin.x, 0);
-    ensure_equals(origin.y, 0);
-    ensure_equals(origin.z, 0);
-    ensure(with_origin.intersects(origin));
-
-    ensure("empty envelope does not intersect coordinate", !empty.intersects(origin));
+    ensure_no_fp_except();
 }
 
-// Test of expand()
+// Test of expandToInclude()
 template<>
 template<>
 void object::test<8>
@@ -238,9 +331,11 @@ void object::test<8>
     empty.expandToInclude(&box);
     ensure("expanding null envelope to include non-null envelope makes null envelope not null",
            empty == exemplar);
+
+    ensure_no_fp_except();
 }
 
-// Second test of expand()
+// Second test of expandToInclude()
 template<>
 template<>
 void object::test<9>
@@ -257,6 +352,8 @@ void object::test<9>
     // Expand empty envelope to include bigger envelope
     empty.expandToInclude(&box);
     ensure(empty == exemplar);
+
+    ensure_no_fp_except();
 }
 
 // Test point-to-envelope distance
@@ -278,11 +375,9 @@ void object::test<10>
     // 20 21 22 23 24
     std::vector<Coordinate> c(25);
 
-    // std::cout<<std::endl;
     for (std::size_t i = 0; i < c.size(); i++) {
         c[i].x = static_cast<double>(i % 5);
         c[i].y = static_cast<double>(5 - (i / 5));
-        // std::cout<< c[i] << std::endl;
     }
 
     // point contained in envelope
@@ -319,6 +414,7 @@ void object::test<10>
     ensure_equals(Envelope::distanceToCoordinate(c[6], c[12], c[14]),
                   c[6].distance(c[12]));
 
+    ensure_no_fp_except();
 }
 
 // Test envelope distance
@@ -371,6 +467,177 @@ void object::test<11>
     b = Envelope({17, 3}, {20, 5});
     ensure_equals(a.distance(b), Coordinate(13, 11).distance(Coordinate(17, 5)));
     ensure_equals(a.distance(b), b.distance(a));
+
+    ensure_no_fp_except();
+}
+
+// comparison of empty envelopes
+template<>
+template<>
+void object::test<12>
+()
+{
+    Envelope empty1;
+    Envelope empty2;
+
+    ensure(!(empty1 < empty2));
+    ensure(!(empty2 < empty1));
+
+    ensure_no_fp_except();
+}
+
+// Envelope::intersects(Coordinate, Coordinate)
+template<>
+template<>
+void object::test<13>
+()
+{
+    Envelope empty;
+
+    ensure(!empty.intersects({1, 1}, {2, 2}));
+
+    ensure_no_fp_except();
+}
+
+// Test of expandBy
+template<>
+template<>
+void object::test<14>
+()
+{
+    // expanding null envelope is still null
+    Envelope empty;
+    empty.expandBy(10, 10);
+    ensure(empty.isNull());
+
+    // expanding a regular envelope gives expected result
+    Envelope e(0, 1, -2, 2);
+    e.expandBy(2, 1);
+    ensure_equals(e.getMinX(), -2);
+    ensure_equals(e.getMaxX(), 3);
+    ensure_equals(e.getMinY(), -3);
+    ensure_equals(e.getMaxY(), 3);
+
+    // expanding envelope by negative amount shrinks it
+    e.expandBy(-2, -1);
+    ensure_equals(e, Envelope(0, 1, -2, 2));
+
+    // shrinking it until it disappears makes it null
+    e.expandBy(-100, -100);
+    ensure(e.isNull());
+
+    ensure_no_fp_except();
+}
+
+// Test of intersection
+template<>
+template<>
+void object::test<15>
+()
+{
+    Envelope a(0, 1, 0, 1);
+    Envelope b(1, 2, 1, 1);
+    Envelope c;
+    Envelope d(100, 200, 100, 200);
+
+    // A - B
+    Envelope ab_intersection;
+    bool ab_intersect = a.intersection(b, ab_intersection);
+    ensure(ab_intersect);
+    ensure_equals(ab_intersection, Envelope(1, 1, 1, 1));
+
+    // A - C
+    Envelope ac_intersection;
+    bool ac_intersect = a.intersection(c, ac_intersection);
+    ensure(!ac_intersect);
+    ensure(ac_intersection.isNull());
+
+    // A - D
+    Envelope ad_intersection;
+    bool ad_intersect = a.intersection(c, ad_intersection);
+    ensure(!ad_intersect);
+    ensure(ad_intersection.isNull());
+
+    // B - C
+    Envelope bc_intersection;
+    bool bc_intersect = a.intersection(c, bc_intersection);
+    ensure(!bc_intersect);
+    ensure(bc_intersection.isNull());
+
+    ensure_no_fp_except();
+}
+
+// Test of centre
+template<>
+template<>
+void object::test<16>
+()
+{
+    // regular envelope
+    Envelope e(0, 1, 2, 4);
+    Coordinate c;
+
+    bool success = e.centre(c);
+    ensure(success);
+    ensure_equals(c, Coordinate{0.5, 3});
+
+    // null envelope
+    Envelope empty;
+    success = empty.centre(c);
+    ensure(!success);
+    ensure_equals(c, Coordinate{0.5, 3});
+
+    ensure_no_fp_except();
+}
+
+// Test of translate
+template<>
+template<>
+void object::test<17>
+()
+{
+    // regular envelope
+    Envelope e(0, 1, 2, 4);
+    e.translate(1, 2);
+    ensure_equals(e, Envelope(1, 2, 4, 6));
+
+    // null envelope
+    Envelope empty;
+    empty.translate(1, 2);
+    ensure(empty.isNull());
+
+    ensure_no_fp_except();
+}
+
+// Test of hashCode
+template<>
+template<>
+void object::test<18>
+()
+{
+    Envelope a(0, 1, 2, 3);
+    Envelope b;
+
+    ensure(a.hashCode() != b.hashCode());
+
+    ensure_no_fp_except();
+}
+
+// Test of expandToInclude(Coordinate)
+template<>
+template<>
+void object::test<19>
+()
+{
+    Envelope e;
+    e.expandToInclude({6, 7});
+
+    ensure_equals(e, Envelope(6, 6, 7, 7));
+
+    e.expandToInclude(0, 1);
+    ensure_equals(e, Envelope(0, 6, 1, 7));
+
+    ensure_no_fp_except();
 }
 
 } // namespace tut
